(*****************************************************************************
 * HOL-TestGen --- theorem-prover based test case generation
 *                 http://www.brucker.ch/projects/hol-testgen/
 *                                                                            
 * --- 
 * This file is part of HOL-TestGen.
 *
 * Copyright (c) 2005-2010 ETH Zurich, Switzerland
 *               2008-2010 Achim D. Brucker, Germany 
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *
 *     * Redistributions in binary form must reproduce the above
 *       copyright notice, this list of conditions and the following
 *       disclaimer in the documentation and/or other materials provided
 *       with the distribution.
 *
 *     * Neither the name of the copyright holders nor the names of its
 *       contributors may be used to endorse or promote products derived
 *       from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ******************************************************************************)
(* $Id: networks.thy 8881 2010-04-02 09:37:05Z brucker $ *)

theory 
  networks
imports
  FWTesting
begin

profiling_on

constdefs
   subnet     :: "IntegerPort net"
  "subnet     \<equiv> {{(a,b) . (a<12) }}"

  "subnet_01" ::"IntegerPort net"
  "subnet_01  \<equiv>  {{(a,b) . ( 0<= a) \<and> (a< 8) }}"

  "subnet_02" ::"IntegerPort net"
  "subnet_02  \<equiv>  {{(a,b) . ( 8<= a) \<and> (a<16) }}"

  "subnet_03" ::"IntegerPort net"
  "subnet_03  \<equiv>  {{(a,b) . (16<= a) \<and> (a<32) }}"

  "subnet_04" ::"IntegerPort net"
  "subnet_04  \<equiv>  {{(a,b) . (32<= a) \<and> (a<40) }}"

  "subnet_05" ::"IntegerPort net"
  "subnet_05  \<equiv>  {{(a,b) . (40<= a) \<and> (a<48) }}"

  "subnet_06" ::"IntegerPort net"
  "subnet_06  \<equiv>  {{(a,b) . (48<= a) \<and> (a<56) }}"

  "subnet_07" ::"IntegerPort net"
  "subnet_07  \<equiv>  {{(a,b) . (56<= a) \<and> (a<64) }}"

  "subnet_08" ::"IntegerPort net"
  "subnet_08  \<equiv>  {{(a,b) . (64<= a) \<and> (a<72) }}"

  "subnet_09" ::"IntegerPort net"
  "subnet_09  \<equiv>  {{(a,b) . (72<= a) \<and> (a<80) }}"

  "subnet_10" ::"IntegerPort net"
  "subnet_10  \<equiv>  {{(a,b) . (80<= a) \<and> (a<88) }}"

  "internet" ::"IntegerPort net"
  "internet  \<equiv>  {{(a,b) . (12>= a)}}"

constdefs
 not_in_same_net :: "(IntegerPort,DummyContent) packet \<Rightarrow> bool"
 "not_in_same_net x  \<equiv>  (src x \<sqsubset> subnet \<longrightarrow> ( \<not> (dest x \<sqsubset> subnet))) \<and> 
                        (src x \<sqsubset> internet \<longrightarrow>  \<not> dest x \<sqsubset> internet)"

constdefs 
  policy_00 :: "(IntegerPort, DummyContent) Policy"
 "policy_00 \<equiv> deny_all"

  policy_01 :: "(IntegerPort, DummyContent) Policy"
 "policy_01 \<equiv> policy_00 ++ allow_from_port_to (80::port) subnet_01 internet"

  policy_02 :: "(IntegerPort, DummyContent) Policy"
 "policy_02 \<equiv> policy_01 ++ allow_from_port_to (80::port) subnet_02 internet"

  policy_03 :: "(IntegerPort, DummyContent) Policy"
 "policy_03 \<equiv> policy_02 ++ allow_from_port_to (80::port) subnet_03 internet"

  policy_04 :: "(IntegerPort, DummyContent) Policy"
 "policy_04 \<equiv> policy_03 ++ allow_from_port_to (80::port) subnet_04 internet"

  policy_05 :: "(IntegerPort, DummyContent) Policy"
 "policy_05 \<equiv> policy_04 ++ allow_from_port_to (80::port) subnet_05 internet"

  policy_06 :: "(IntegerPort, DummyContent) Policy"
 "policy_06 \<equiv> policy_05 ++ allow_from_port_to (80::port) subnet_06 internet"

  policy_07 :: "(IntegerPort, DummyContent) Policy"
 "policy_07 \<equiv> policy_06 ++ allow_from_port_to (80::port) subnet_07 internet"

  policy_08 :: "(IntegerPort, DummyContent) Policy"
 "policy_08 \<equiv> policy_07 ++ allow_from_port_to (80::port) subnet_08 internet"

  policy_09 :: "(IntegerPort, DummyContent) Policy"
 "policy_09 \<equiv> policy_08 ++ allow_from_port_to (80::port) subnet_09 internet"

  policy_10 :: "(IntegerPort, DummyContent) Policy"
 "policy_10 \<equiv> policy_09 ++ allow_from_port_to (80::port) subnet_10 internet"

lemmas UnfoldPolicy = policy_00_def policy_01_def policy_02_def policy_03_def policy_04_def 
                      policy_05_def policy_06_def policy_07_def policy_08_def policy_09_def 
                      policy_10_def

lemmas UnfoldNetworks = subnet_def internet_def subnet_01_def subnet_02_def subnet_03_def
                        subnet_04_def subnet_05_def subnet_06_def subnet_07_def subnet_08_def 
                        subnet_09_def subnet_10_def

lemmas UnfoldAll = UnfoldPolicy  UnfoldNetworks not_in_same_net_def PolicyCombinators PortCombinators
                   src_def dest_def IntegerPortLemmas in_subnet_def content_def


test_spec "not_in_same_net x \<and> content x = data \<longrightarrow> FUT x = policy_00 x"
apply (prepare_fw_spec)
apply (simp add: UnfoldAll)
  apply (gen_test_cases "FUT", simp_all add: UnfoldAll)
store_test_thm "policy_00"


testgen_params[iterations=100]


gen_test_data "policy_00"

test_spec "not_in_same_net x \<longrightarrow> FUT x = policy_01 x"
apply (prepare_fw_spec)
  apply (simp add: UnfoldAll)
  apply (gen_test_cases "FUT", simp_all add: UnfoldAll)
store_test_thm "policy_01"


gen_test_data "policy_01"

test_spec "not_in_same_net x \<longrightarrow> FUT x = policy_02 x"
apply (prepare_fw_spec)
  apply (simp add: UnfoldAll)
  apply (gen_test_cases "FUT", simp_all add: UnfoldAll)
store_test_thm "policy_02"


gen_test_data "policy_02"

test_spec "not_in_same_net x \<longrightarrow> FUT x = policy_03 x"
apply (prepare_fw_spec)
  apply (simp add: UnfoldAll)
  apply (gen_test_cases "FUT")  
apply (simp_all add: UnfoldAll)
store_test_thm "policy_03"


gen_test_data "policy_03"

test_spec "not_in_same_net x \<longrightarrow> FUT x = policy_04 x"
apply (prepare_fw_spec)
  apply (simp add: UnfoldAll)
  apply (gen_test_cases "FUT", simp_all add: UnfoldAll)
store_test_thm "policy_04"


gen_test_data "policy_04"

test_spec "not_in_same_net x \<longrightarrow> FUT x = policy_05 x"
apply (prepare_fw_spec)
  apply (simp add: UnfoldAll)
  apply (gen_test_cases "FUT", simp_all add: UnfoldAll)
store_test_thm "policy_05"


gen_test_data "policy_05"

test_spec "not_in_same_net x \<longrightarrow> FUT x = policy_06 x"
apply (prepare_fw_spec)
 apply (simp add: UnfoldAll)
  apply (gen_test_cases "FUT", simp_all add: UnfoldAll)
store_test_thm "policy_06"


gen_test_data "policy_06"

test_spec "not_in_same_net x \<longrightarrow> FUT x = policy_07 x"
apply (prepare_fw_spec)
  apply (simp add: UnfoldAll)
  apply (gen_test_cases "FUT", simp_all add: UnfoldAll)
store_test_thm "policy_07"


gen_test_data "policy_07"

test_spec "not_in_same_net x \<longrightarrow> FUT x = policy_08 x"
apply (prepare_fw_spec)
  apply (simp add: UnfoldAll)
  apply (gen_test_cases "FUT", simp_all add: UnfoldAll)
store_test_thm "policy_08"


gen_test_data "policy_08"

test_spec "not_in_same_net x \<longrightarrow> FUT x = policy_09 x"
apply (prepare_fw_spec)
  apply (simp add: UnfoldAll)
  apply (gen_test_cases "FUT", simp_all add: UnfoldAll)
store_test_thm "policy_09"


gen_test_data "policy_09"

test_spec "not_in_same_net x \<longrightarrow> FUT x = policy_10 x"
apply (prepare_fw_spec)
  apply (simp add: UnfoldAll)
  apply (gen_test_cases "FUT", simp_all add: UnfoldAll)
store_test_thm "policy_10"



gen_test_data "policy_10"


write_clocks "time_stats_networks.tex"

profiling_off


end
