(*****************************************************************************
 * HOL-TestGen --- theorem-prover based test case generation
 *                 http://www.brucker.ch/projects/hol-testgen/
 *                                                                            
 * --- 
 * This file is part of HOL-TestGen.
 *
 * Copyright (c) 2005-2010 ETH Zurich, Switzerland
 *               2008-2010 Achim D. Brucker, Germany 
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *
 *     * Redistributions in binary form must reproduce the above
 *       copyright notice, this list of conditions and the following
 *       disclaimer in the documentation and/or other materials provided
 *       with the distribution.
 *
 *     * Neither the name of the copyright holders nor the names of its
 *       contributors may be used to endorse or promote products derived
 *       from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ******************************************************************************)
(* $Id: rules-DatatypeNetwork.thy 8881 2010-04-02 09:37:05Z brucker $ *)

theory 
  "rules-DatatypeNetwork"
imports
  FWTesting
begin

profiling_on

datatype Adr = Workstation | Internet
instance Adr :: adr .. 
types port = int
types Networks = "Adr\<times>port"


constdefs
  workstation::"Networks net"
  "workstation \<equiv> {{(a,b). a= Workstation}}"
  internet :: "Networks net"
  "internet \<equiv> {{(a,b). a= Internet}}"

defs (overloaded)
 src_port_def:  "src_port (x::(Networks,'b) packet)  \<equiv> (snd o fst o snd) x"
 dest_port_def: "dest_port (x::(Networks,'b) packet)  \<equiv> (snd o fst o snd o snd) x"
 subnet_of_def: "subnet_of (x::Networks) \<equiv> {{(a,b). a = fst x}}" 


constdefs
 not_in_same_net :: "(Networks,DummyContent) packet \<Rightarrow> bool"
 "not_in_same_net x  \<equiv>  (src x \<sqsubset> workstation \<longrightarrow> ( (dest x \<sqsubset> internet))) \<and> 
                        (src x \<sqsubset> internet \<longrightarrow>  dest x \<sqsubset> workstation)"

                     
constdefs
   policy_00 :: "(Networks, DummyContent) Policy"
  "policy_00 \<equiv> deny_all"

   policy_01 :: "(Networks, DummyContent) Policy"
  "policy_01 \<equiv> policy_00 ++ allow_from_port_to (80::port) workstation internet"

   policy_02 :: "(Networks, DummyContent) Policy"
  "policy_02 \<equiv> policy_01 ++ allow_from_port_to (21::port) workstation internet"

   policy_03 :: "(Networks, DummyContent) Policy"
  "policy_03 \<equiv> policy_02 ++ allow_from_port_to (56::port) workstation internet"

   policy_04 :: "(Networks, DummyContent) Policy"
  "policy_04 \<equiv> policy_03 ++ allow_from_port_to (34::port) workstation internet "

   policy_05 :: "(Networks, DummyContent) Policy"
  "policy_05 \<equiv> policy_04 ++ allow_from_port_to (13::port) workstation internet"

   policy_06 :: "(Networks, DummyContent) Policy"
  "policy_06 \<equiv> policy_05 ++ allow_from_port_to (456::port) workstation internet"

   policy_07 :: "(Networks, DummyContent) Policy"
  "policy_07 \<equiv> policy_06 ++ allow_from_port_to (443::port) workstation internet"

   policy_08 :: "(Networks, DummyContent) Policy"
  "policy_08 \<equiv> policy_07 ++ allow_from_port_to (21::port) workstation internet"

   policy_09 :: "(Networks, DummyContent) Policy"
  "policy_09 \<equiv> policy_08 ++ allow_from_port_to (443::port) workstation internet"

   policy_10 :: "(Networks, DummyContent) Policy"
  "policy_10 \<equiv> policy_09 ++ allow_from_port_to (342::port) workstation internet"

lemmas UnfoldPolicy =  src_def dest_def in_subnet_def 
                       workstation_def internet_def policy_00_def 
                       policy_01_def policy_02_def policy_03_def policy_04_def policy_05_def
                       policy_06_def policy_07_def policy_08_def policy_09_def policy_10_def

lemmas PolicyLemmas =  UnfoldPolicy workstation_def internet_def 
                       src_def dest_def src_port_def dest_port_def in_subnet_def

test_spec "not_in_same_net x \<longrightarrow> FUT x = policy_00 x"
apply (prepare_fw_spec)
apply (simp add: PolicyLemmas PortCombinators PolicyCombinators)
apply (gen_test_cases "FUT")
apply (simp_all add: not_in_same_net_def PolicyLemmas)
store_test_thm "policy_00"

gen_test_data "policy_00"
thm policy_00.test_data

test_spec "not_in_same_net x \<longrightarrow> FUT x = policy_01 x"
apply (prepare_fw_spec)
apply (simp add: PolicyLemmas PortCombinators PolicyCombinators)
apply (gen_test_cases "FUT")
apply (simp_all add: not_in_same_net_def PolicyLemmas)
store_test_thm "policy_01"

gen_test_data"policy_01"
thm policy_01.test_data

test_spec"not_in_same_net x \<longrightarrow> FUT x = policy_02 x"
apply (prepare_fw_spec)
apply (simp add: PolicyLemmas PortCombinators PolicyCombinators)
apply (gen_test_cases"FUT")
apply (simp_all add: not_in_same_net_def PolicyLemmas)
store_test_thm"policy_02"

gen_test_data"policy_02"
thm policy_02.test_data

test_spec"not_in_same_net x \<longrightarrow> FUT x = policy_03 x"
apply (prepare_fw_spec)
apply (simp add: PolicyLemmas PortCombinators PolicyCombinators)
apply (gen_test_cases"FUT")
apply (simp_all add: not_in_same_net_def PolicyLemmas)
store_test_thm"policy_03"

gen_test_data"policy_03"
thm policy_03.test_data

test_spec"not_in_same_net x \<longrightarrow> FUT x = policy_04 x"
apply (prepare_fw_spec)
apply (simp add: PolicyLemmas PortCombinators PolicyCombinators)
apply (gen_test_cases"FUT")
apply (simp_all add: not_in_same_net_def PolicyLemmas)
store_test_thm"policy_04"

gen_test_data"policy_04"
thm policy_04.test_data

test_spec"not_in_same_net x \<longrightarrow> FUT x = policy_05 x"
apply (prepare_fw_spec)
apply (simp add: PolicyLemmas PortCombinators PolicyCombinators)
apply (gen_test_cases"FUT")
apply (simp_all add: not_in_same_net_def PolicyLemmas)
store_test_thm"policy_05"

gen_test_data"policy_05"
thm policy_05.test_data

test_spec"not_in_same_net x \<longrightarrow> FUT x = policy_06 x"
apply (prepare_fw_spec)
apply (simp add: PolicyLemmas PortCombinators PolicyCombinators)
apply (gen_test_cases"FUT")
apply (simp_all add: not_in_same_net_def PolicyLemmas)
store_test_thm"policy_06"

gen_test_data"policy_06"
thm policy_06.test_data

test_spec"not_in_same_net x \<longrightarrow> FUT x = policy_07 x"
apply (prepare_fw_spec)
apply (simp add: PolicyLemmas PortCombinators PolicyCombinators)
apply (gen_test_cases"FUT")
apply (simp_all add: not_in_same_net_def PolicyLemmas)
store_test_thm"policy_07"

gen_test_data"policy_07"
thm policy_07.test_data

test_spec"not_in_same_net x \<longrightarrow> FUT x = policy_08 x"
apply (prepare_fw_spec)
apply (simp add: PolicyLemmas PortCombinators PolicyCombinators)
apply (gen_test_cases"FUT")
apply (simp_all add: not_in_same_net_def PolicyLemmas)
store_test_thm"policy_08"

gen_test_data"policy_08"
thm policy_08.test_data

test_spec"not_in_same_net x \<longrightarrow> FUT x = policy_09 x"
apply (prepare_fw_spec)
apply (simp add: PolicyLemmas PortCombinators PolicyCombinators)
apply (gen_test_cases"FUT")
apply (simp_all add: not_in_same_net_def PolicyLemmas)
store_test_thm"policy_09"

gen_test_data"policy_09"
thm policy_0.test_data

test_spec"not_in_same_net x \<longrightarrow> FUT x = policy_10 x"
apply (prepare_fw_spec)
apply (simp add: PolicyLemmas PortCombinators PolicyCombinators)
apply (gen_test_cases"FUT")
apply (simp_all add: not_in_same_net_def PolicyLemmas)
store_test_thm"policy_10"

gen_test_data"policy_10"
thm policy_10.test_data


write_clocks "time_stats_rulesDatatypeNetwork.tex"

profiling_off


end
