#
# Author: Sascha Boehme, TU Muenchen
#
# Cache for prover results, based on discriminating problems using MD5.

use strict;
use warnings;
use Digest::MD5;


# arguments

my $prefix = shift @ARGV;
my ($index_db, $result_db) = ("$prefix.index", "$prefix.result");
my $result_file = pop @ARGV;
my $problem_file = $ARGV[-1];


# create MD5 problem digest

my $md5 = Digest::MD5->new;
open FILE, "<$problem_file" or
  die "ERROR: Failed to open '$problem_file' ($!)";
$md5->addfile(*FILE);
close FILE;
my $problem_digest = $md5->b64digest;


# lookup problem digest in MD5 index

my $result_digest = undef;

if (-e $index_db) {
  open DB, "<$index_db" or die "ERROR: Failed to open '$index_db' ($!)";
  foreach (<DB>) {
    /(\w+) (\w+)/;
    if ($problem_digest eq $1) {
      defined $result_digest and die "ERROR: MD5 collision";
      $result_digest = $2;
    }
  }
  close DB;
}


# print prover result (obtained from the cache or the prover)

if (defined $result_digest) {
  open DB, "<$result_db" or die "ERROR: Failed to open '$result_db' ($!)";
  open FILE, ">$result_file" or exit 0;
  my ($skip, $print) = (0, 0);
  foreach (<DB>) {
    if ($skip == 0) {
      if ($print != 0) { last; }
      /(\w+) (\d+)/;
      if ($1 eq $result_digest) { $print = 1; }
      $skip = $2;
    }
    elsif ($print != 0) { print FILE $_; }
  }
  close FILE;
  close DB;
}
else {
  system "@ARGV >$result_file 2>&1";

  if (-e $result_file) {
    open FILE, "<$result_file" or exit 0;
    $md5->reset;
    my $lines = 0;
    foreach (<FILE>) {
      $lines++;
      $md5->add($_);
    }
    $result_digest = $md5->b64digest;
    seek FILE, 0, 0;
    open DB, ">>$result_db" or die "ERROR: Failed to open '$result_db' ($!)";
    seek DB, 0, 2;
    print DB "$result_digest $lines\n";
    foreach (<FILE>) {
      $md5->add($_);
      print DB $_;
    }
    close DB;
    close FILE;
  }
  

  open DB, ">>$index_db" or die "ERROR: Unable to open '$index_db' ($!)";
  print DB "$problem_digest $result_digest\n";
  close DB;
}

