(*****************************************************************************
 * HOL-TestGen --- theorem-prover based test case generation
 *                 http://www.brucker.ch/projects/hol-testgen/
 *                                                                            
 * TestEnv.ML  --- environment for configuration parameters
 * This file is part of HOL-TestGen.
 *
 * Copyright (c) 2005-2007 ETH Zurich, Switzerland
 *               2008 Achim D. Brucker, Germanycat 
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *
 *     * Redistributions in binary form must reproduce the above
 *       copyright notice, this list of conditions and the following
 *       disclaimer in the documentation and/or other materials provided
 *       with the distribution.
 *
 *     * Neither the name of the copyright holders nor the names of its
 *       contributors may be used to endorse or promote products derived
 *       from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ******************************************************************************)
     
infix 4 add_test_case  del_test_case  add_test_data  del_test_data 

signature TESTENV =
sig
   type testinfo =
               {depth   : int, (* data structure depth *)
                breadth : int, (* no of free's in testspec *)
                test_thm_tab    : thm Symtab.table, (* name to testthm *)

                bound           : int, (* global bound for data statements *)
                case_breadth    : int, (* no of test data per case,
                                          weakening uniformity *)
                iterations      : int, (* number of random attempts to solve a case *)

                type_range_bound: int,      (* effectively used elements 
                                               type grounding list *)
                type_candicates : typ list, (* type grounding list *)
 
                cod_term_tab    : (typ list -> term) Symtab.table,
                                       (* random generators for terms,
                                          tab assigns to each type constuctor 
                                          name a generator that gets the
                                          instances of type constructor as args.*)
                abstest_data_tab: (thm list) Symtab.table, 
                                       (* assigns name to list of 
                                          abstract (local) test data statements *)
                test_data_tab   : (thm list) Symtab.table,

                pre_safe_tac           :  tactic,
                pre_normalize_TNF_tac  :  tactic,
                pre_minimize_TNF_tac   :  tactic
 };


   type testenv
  
   val  rep_testenv       : testenv -> testinfo
   val  mk_testenv        : testinfo -> testenv
 
   val  add_test_case     : testenv * (string*thm)  -> testenv
   (*   add test theorem of "name".
        The test-theorem is assumed to consist of either clauses in TNF or 
        of Test Hypotheses. *)
       
   val  del_test_case     : testenv * string -> testenv
   val  add_test_data     : testenv * (string * thm)  -> testenv
   (* used to insert abstract test data registrated under
      "string" into the test environment; abstract test data
      (or "local test theorems") were used in gen_test_data *)
   val  del_test_data     : testenv * string  -> testenv
   (* erase abstract test data from test environment *)

   val  merge_testenv     : testenv * testenv -> testenv

   val export_test_data   : theory ->string -> string -> testenv -> theory 
 
   val get_depth          : testenv -> int 
   val get_breadth        : testenv -> int
   val get_gen_prelude    : testenv -> bool 
   val get_gen_wrapper    : testenv -> bool 
   val get_SMT            : testenv ->bool
   val get_gen_toString   : testenv -> string 
   val get_pre_safe_tac   : testenv ->  tactic
   val get_normalizeTNF_tac : testenv -> tactic
   val get_minimizeTNF_tac  : testenv -> tactic

   val depth_update       : int  -> testenv -> testenv
   val setup_code_update  : string  -> testenv -> testenv
   val dataconv_code_update  : string  -> testenv -> testenv
   val breadth_update     : int  -> testenv   -> testenv
   val bound_update       : int  -> testenv   -> testenv
   val case_breadth_update: int  -> testenv -> testenv
   val iterations_update  : int  -> testenv -> testenv
   val gen_prelude_update : bool  -> testenv -> testenv
   val gen_wrapper_update : bool  -> testenv -> testenv
   val SMT_update         : bool  -> testenv -> testenv
   val gen_toString_update : string  -> testenv -> testenv
   val type_range_bound_update : int  -> testenv -> testenv
   val type_candiates_update   : typ list -> testenv -> testenv
   val pre_safe_tac_update     :  tactic -> testenv -> testenv
   val pre_normalizeTNF_tac_update     : tactic -> testenv -> testenv
   val pre_minimizeTNF_tac_update      : tactic -> testenv -> testenv
end

structure TestEnv (*  : TESTENV *) =
struct

open HOLogic;

type testinfo ={depth           : int, (* data structure depth *)
                breadth         : int, (* no of free's in testspec *)
                test_thm_tab    : thm Symtab.table, (* name to testthm *)

                bound           : int, (* global bound for data statements *)
                case_breadth    : int, (* no of test data per case,
                                          weakening uniformity *)
                iterations      : int, (* number of random attempts to solve a case *)

                type_range_bound: int,      (* effectively used elements 
                                               type grounding list *)
                type_candicates : typ list, (* type grounding list *)
 
		prelude         : bool,
		wrapper         : bool,
		smt             : bool,
		toString        : string,
		setupCode       : string,
		dataconvCode    : string,
                cod_term_tab    : (typ list -> term) Symtab.table,
                                    (* random generators for terms,
                                       tab assigns to each type constuctor 
                                       name a generator that gets the
                                       instances of type constructor as args.*)
                abstest_data_tab: (thm list) Symtab.table, 
                                    (* assigns name to list of 
                                       data statements *)
                test_data_tab   : (thm list) Symtab.table,
                pre_safe_tac           :  Proof.context -> clasimpset -> tactic, 
                pre_normalizeTNF_tac   :  Proof.context -> clasimpset -> tactic,  
                pre_minimizeTNF_tac    :  Proof.context -> clasimpset -> tactic
                };


datatype testenv   = Testenv of testinfo

fun rep_testenv (Testenv X) = X;
fun mk_testenv  (X:testinfo)= Testenv X

fun (Testenv {depth,breadth,test_thm_tab,
              bound,case_breadth,iterations,type_range_bound,
              type_candicates,prelude,wrapper,smt,toString,setupCode,dataconvCode,
	      cod_term_tab, abstest_data_tab,test_data_tab,pre_safe_tac,pre_normalizeTNF_tac,pre_minimizeTNF_tac}) 
	add_test_case 
    (name,thm)   =
    Testenv({depth=depth,
             breadth=breadth,
             test_thm_tab=Symtab.update(name,thm)test_thm_tab,
             bound=bound,case_breadth=case_breadth,
             iterations=iterations,type_range_bound=type_range_bound,
             type_candicates=type_candicates,prelude=prelude,wrapper=wrapper,smt=smt,toString=toString,setupCode=setupCode,
             dataconvCode=dataconvCode,cod_term_tab=cod_term_tab, 
             abstest_data_tab=abstest_data_tab,
             test_data_tab=test_data_tab, pre_safe_tac=pre_safe_tac,pre_normalizeTNF_tac=pre_normalizeTNF_tac,
             pre_minimizeTNF_tac=pre_minimizeTNF_tac});


(* inefficient !!! *)

fun(Testenv {depth,breadth,test_thm_tab,
             bound,case_breadth,iterations,type_range_bound,
             type_candicates,prelude,wrapper,smt,toString,setupCode,dataconvCode,cod_term_tab, abstest_data_tab,
             test_data_tab,pre_safe_tac,pre_normalizeTNF_tac,pre_minimizeTNF_tac}) 
    del_test_case 
    name  =
    Testenv({depth=depth,
             breadth=breadth,
             test_thm_tab=Symtab.make(filter (fn(x,_) => not(x=name)) 
                                             (Symtab.dest test_thm_tab)),
             bound=bound,case_breadth=case_breadth,
             iterations=iterations,type_range_bound=type_range_bound,
             type_candicates=type_candicates,prelude=prelude,wrapper=wrapper,
             smt=smt,toString=toString,dataconvCode=dataconvCode,setupCode=setupCode,
             cod_term_tab=cod_term_tab, abstest_data_tab=abstest_data_tab,
             test_data_tab=test_data_tab, pre_safe_tac=pre_safe_tac,pre_normalizeTNF_tac=pre_normalizeTNF_tac,
             pre_minimizeTNF_tac=pre_minimizeTNF_tac});




fun(Testenv {depth,breadth,test_thm_tab,
             bound,case_breadth,iterations,type_range_bound,
             type_candicates,prelude,wrapper,smt,toString,setupCode,dataconvCode,
	     cod_term_tab, abstest_data_tab,test_data_tab,pre_safe_tac,pre_normalizeTNF_tac,pre_minimizeTNF_tac}) 
    add_test_data 
    (name,thm) =
    Testenv({depth=depth,
             breadth=breadth,
             test_thm_tab=test_thm_tab,
             bound=bound,case_breadth=case_breadth,
             iterations=iterations,type_range_bound=type_range_bound,
             type_candicates=type_candicates,prelude=prelude,wrapper=wrapper,
	     smt=smt,toString=toString,setupCode=setupCode,dataconvCode=dataconvCode,
             cod_term_tab=cod_term_tab, 
             abstest_data_tab=Symtab.update
                           (name, thm::(case Symtab.lookup abstest_data_tab name of
                                         NONE => []
                                       | SOME S => S))
                            abstest_data_tab,
             test_data_tab=test_data_tab,pre_safe_tac=pre_safe_tac,pre_normalizeTNF_tac=pre_normalizeTNF_tac,
             pre_minimizeTNF_tac=pre_minimizeTNF_tac});


fun(Testenv {depth,breadth,test_thm_tab,
             bound,case_breadth,iterations,type_range_bound,
             type_candicates,prelude,wrapper,smt,toString,setupCode,dataconvCode,
	     cod_term_tab,abstest_data_tab,test_data_tab,pre_safe_tac,pre_normalizeTNF_tac,pre_minimizeTNF_tac}) 
       del_test_data 
    name  =
    Testenv({depth=depth,
             breadth=breadth,
             test_thm_tab=test_thm_tab,
             bound=bound,case_breadth=case_breadth,
             iterations=iterations,type_range_bound=type_range_bound,
             type_candicates=type_candicates,prelude=prelude,wrapper=wrapper,
	     smt=smt,toString=toString,dataconvCode=dataconvCode,setupCode=setupCode,
             cod_term_tab=cod_term_tab, 
             abstest_data_tab=Symtab.make(filter (fn(x,_) => not(x=name)) 
                                        (Symtab.dest abstest_data_tab)),
             test_data_tab=test_data_tab,pre_safe_tac=pre_safe_tac,pre_normalizeTNF_tac=pre_normalizeTNF_tac,
             pre_minimizeTNF_tac=pre_minimizeTNF_tac});


fun merge_testenv
   (Testenv{depth=dd,     breadth=db,
            test_thm_tab=tttab,   bound=b,
            case_breadth=cb,      iterations=i,
            type_range_bound=trb, type_candicates=tc,prelude=p,wrapper=w,smt=sm,toString=s,
            cod_term_tab=ctt,     abstest_data_tab=tdt,dataconvCode=dc,setupCode=sc,
            test_data_tab=tjt,pre_safe_tac=pst,pre_normalizeTNF_tac=pnt,pre_minimizeTNF_tac=pmt},
    Testenv{depth=dd',    breadth=db',
            test_thm_tab=tttab',  bound=b',
            case_breadth=cb',     iterations=i',dataconvCode=dc',setupCode=sc',
            type_range_bound=trb',type_candicates=tc',prelude=p',wrapper=w',smt=sm',toString=s',
            cod_term_tab=ctt',    abstest_data_tab=tdt',
            test_data_tab=tjt',pre_safe_tac=pst',pre_normalizeTNF_tac=pnt',pre_minimizeTNF_tac=pmt'}) =

    Testenv{depth         = Int.min(dd,dd'),
            breadth       = Int.min(dd,db'),
	    test_thm_tab  = Symtab.merge (Thm.eq_thm) (tttab,tttab'), 
            bound                 = Int.min(b,b'),
            case_breadth          = Int.min(cb,cb'),
            iterations            = Int.min(i,i'),
            type_range_bound      = Int.min(trb,trb'),
	    type_candicates       = distinct (op=) (tc@tc'),
	    prelude               = p andalso p',
	    wrapper               = w andalso w',
	    smt                   = sm andalso sm',
	    toString               = "" ,
	    setupCode             =sc^"\n"^sc',
	    dataconvCode          =dc^"\n"^dc',
            cod_term_tab          = Symtab.empty, 
                                    (* don't know what a senseful
                                       definition of override could
                                       be; therefore constraint to the
                                       simplest and most conservative
                                       one *)
            abstest_data_tab         = Symtab.merge Thm.eq_thms (tdt,tdt'),
            test_data_tab         = Symtab.merge Thm.eq_thms (tjt,tjt'),
            pre_safe_tac          = pst,
            pre_normalizeTNF_tac  = pnt,
            pre_minimizeTNF_tac   = pmt};
              
   


fun thm_tab  te = #test_thm_tab (rep_testenv te)
fun absdata_tab te = #abstest_data_tab(rep_testenv te)
fun jdmt_tab te = #test_data_tab(rep_testenv te)
(* In the sequel we also use the term "test judgement" synonymously
   for "test data statement". *)

fun thm_tab_update
       (Testenv{depth=dd, breadth=db,
            test_thm_tab=tttab,   bound=b,
            case_breadth=cb,      iterations=i,
            type_range_bound=trb, type_candicates=tc, prelude=p,wrapper=w,smt=sm,toString=s,setupCode=sc,
            cod_term_tab=ctt,     abstest_data_tab=tdt,
		dataconvCode=dc,
            test_data_tab=tjt,pre_safe_tac=pst,pre_normalizeTNF_tac=pnt,pre_minimizeTNF_tac=pmt})
       (thm_tab) =
       (Testenv{depth=dd,     breadth=db,
                test_thm_tab=thm_tab, bound=b,
                case_breadth=cb,      iterations=i,
                type_range_bound=trb, type_candicates=tc,prelude=p,wrapper=w,smt=sm,toString=s,setupCode=sc,
                cod_term_tab=ctt,     abstest_data_tab=tdt,
		dataconvCode=dc,
                test_data_tab=tjt,pre_safe_tac=pst,pre_normalizeTNF_tac=pnt,pre_minimizeTNF_tac=pmt});

fun jdmt_tab_update
       (Testenv{depth=dd, breadth=db,
            test_thm_tab=tttab,   bound=b,
            case_breadth=cb,      iterations=i,
            type_range_bound=trb, type_candicates=tc,prelude=p,wrapper=w,smt=sm,toString=s,setupCode=sc,
            cod_term_tab=ctt,     abstest_data_tab=tdt,
		dataconvCode=dc,
            test_data_tab=tjt,pre_safe_tac=pst,pre_normalizeTNF_tac=pnt,pre_minimizeTNF_tac=pmt})
       (jdmt_tab) =
       (Testenv{depth=dd,     breadth=db,
                test_thm_tab=tttab,   bound=b,
                case_breadth=cb,      iterations=i,
                type_range_bound=trb, type_candicates=tc,prelude=p,wrapper=w,smt=sm,toString=s,setupCode=sc,
                cod_term_tab=ctt,     abstest_data_tab=tdt,
		dataconvCode=dc,
                test_data_tab=jdmt_tab,pre_safe_tac=pst,pre_normalizeTNF_tac=pnt,pre_minimizeTNF_tac=pmt});


fun depth_update (depth)
       (Testenv{depth=dd, breadth=db,
            test_thm_tab=tttab,   bound=b,
            case_breadth=cb,      iterations=i,
            type_range_bound=trb, type_candicates=tc,prelude=p,wrapper=w,smt=sm,toString=s,setupCode=sc,
            cod_term_tab=ctt,     abstest_data_tab=tdt,
		dataconvCode=dc,
            test_data_tab=tjt,pre_safe_tac=pst,pre_normalizeTNF_tac=pnt,pre_minimizeTNF_tac=pmt})
        =
       (Testenv{depth=depth,     breadth=db,
                test_thm_tab=tttab,   bound=b,
                case_breadth=cb,      iterations=i,
                type_range_bound=trb, type_candicates=tc,prelude=p,wrapper=w,smt=sm,toString=s,setupCode=sc,
                cod_term_tab=ctt,     abstest_data_tab=tdt,
		dataconvCode=dc,
                test_data_tab=tjt,pre_safe_tac=pst,pre_normalizeTNF_tac=pnt,pre_minimizeTNF_tac=pmt});

fun breadth_update (breadth)
       (Testenv{depth=dd, breadth=db,
            test_thm_tab=tttab,   bound=b,
            case_breadth=cb,      iterations=i,
            type_range_bound=trb, type_candicates=tc,prelude=p,wrapper=w,smt=sm,toString=s,setupCode=sc,
            cod_term_tab=ctt,     abstest_data_tab=tdt,
		dataconvCode=dc,
            test_data_tab=tjt,pre_safe_tac=pst,pre_normalizeTNF_tac=pnt,pre_minimizeTNF_tac=pmt})
        =
       (Testenv{depth=dd,     breadth=breadth,
                test_thm_tab=tttab,   bound=b,
                case_breadth=cb,      iterations=i,
                type_range_bound=trb, type_candicates=tc,prelude=p,wrapper=w,smt=sm,toString=s,setupCode=sc,
                cod_term_tab=ctt,     abstest_data_tab=tdt,
		dataconvCode=dc,
                test_data_tab=tjt,pre_safe_tac=pst,pre_normalizeTNF_tac=pnt,pre_minimizeTNF_tac=pmt});

fun bound_update (bound)
       (Testenv{depth=dd, breadth=db,
                test_thm_tab=tttab,   bound=b,
                case_breadth=cb,      iterations=i,
                type_range_bound=trb, type_candicates=tc,prelude=p,wrapper=w,smt=sm,toString=s,setupCode=sc,
                cod_term_tab=ctt,     abstest_data_tab=tdt,
		dataconvCode=dc,
                test_data_tab=tjt,pre_safe_tac=pst,pre_normalizeTNF_tac=pnt,pre_minimizeTNF_tac=pmt})
        =
       (Testenv{depth=dd,     breadth=db,
                test_thm_tab=tttab,   bound=bound,
                case_breadth=cb,      iterations=i,
                type_range_bound=trb, type_candicates=tc,prelude=p,wrapper=w,smt=sm,toString=s,setupCode=sc,
                cod_term_tab=ctt,     abstest_data_tab=tdt,
		dataconvCode=dc,
                test_data_tab=tjt,pre_safe_tac=pst,pre_normalizeTNF_tac=pnt,pre_minimizeTNF_tac=pmt});

fun case_breadth_update (case_breadth)
       (Testenv{depth=dd, breadth=db,
                test_thm_tab=tttab,   bound=b,
                case_breadth=cb,      iterations=i,
                type_range_bound=trb, type_candicates=tc,prelude=p,wrapper=w,smt=sm,toString=s,setupCode=sc,
                cod_term_tab=ctt,     abstest_data_tab=tdt,
		dataconvCode=dc,
                test_data_tab=tjt,pre_safe_tac=pst,pre_normalizeTNF_tac=pnt,pre_minimizeTNF_tac=pmt})
        =
       (Testenv{depth=dd,     breadth=db,
                test_thm_tab=tttab,   bound=b,
                case_breadth=case_breadth,      iterations=i,
                type_range_bound=trb, type_candicates=tc,prelude=p,wrapper=w,smt=sm,toString=s,setupCode=sc,
                cod_term_tab=ctt,     abstest_data_tab=tdt,
		dataconvCode=dc,
                test_data_tab=tjt,pre_safe_tac=pst,pre_normalizeTNF_tac=pnt,pre_minimizeTNF_tac=pmt});

fun iterations_update (iterations)
       (Testenv{depth=dd, breadth=db,
                test_thm_tab=tttab,   bound=b,
                case_breadth=cb,      iterations=i,
                type_range_bound=trb, type_candicates=tc,prelude=p,wrapper=w,smt=sm,toString=s,setupCode=sc,
                cod_term_tab=ctt,     abstest_data_tab=tdt,
		dataconvCode=dc,
                test_data_tab=tjt,pre_safe_tac=pst,pre_normalizeTNF_tac=pnt,pre_minimizeTNF_tac=pmt})
        =
       (Testenv{depth=dd,     breadth=db,
                test_thm_tab=tttab,   bound=b,
                case_breadth=cb,      iterations=iterations,
                type_range_bound=trb, type_candicates=tc,prelude=p,wrapper=w,smt=sm,toString=s,setupCode=sc,
                cod_term_tab=ctt,     abstest_data_tab=tdt,
		dataconvCode=dc,
                test_data_tab=tjt,pre_safe_tac=pst,pre_normalizeTNF_tac=pnt,pre_minimizeTNF_tac=pmt});

fun type_range_bound_update (type_range_bound)
       (Testenv{depth=dd, breadth=db,
                test_thm_tab=tttab,   bound=b,
                case_breadth=cb,      iterations=i,
                type_range_bound=trb, type_candicates=tc,prelude=p,wrapper=w,smt=sm,toString=s,setupCode=sc,
                cod_term_tab=ctt,     abstest_data_tab=tdt,
		dataconvCode=dc,
                test_data_tab=tjt,pre_safe_tac=pst,pre_normalizeTNF_tac=pnt,pre_minimizeTNF_tac=pmt})
        =
       (Testenv{depth=dd,     breadth=db,
                test_thm_tab=tttab,   bound=b,
                case_breadth=cb,      iterations=i,
                type_range_bound=type_range_bound,type_candicates=tc,
                prelude=p,            wrapper=w,smt=sm,toString=s,setupCode=sc,
                cod_term_tab=ctt,     abstest_data_tab=tdt,
		dataconvCode=dc,
                test_data_tab=tjt,pre_safe_tac=pst,pre_normalizeTNF_tac=pnt,pre_minimizeTNF_tac=pmt});

fun type_candiates_update (type_canditates)
       (Testenv{depth=dd, breadth=db,
                test_thm_tab=tttab,   bound=b,
                case_breadth=cb,      iterations=i,
                type_range_bound=trb, type_candicates=tc,prelude=p,wrapper=w,smt=sm,toString=s,setupCode=sc,
                cod_term_tab=ctt,     abstest_data_tab=tdt,
		dataconvCode=dc,
                test_data_tab=tjt,pre_safe_tac=pst,pre_normalizeTNF_tac=pnt,pre_minimizeTNF_tac=pmt})
        =
       (Testenv{depth=dd,     breadth=db,
                test_thm_tab=tttab,   bound=b,
                case_breadth=cb,      iterations=i,
                type_range_bound=trb, type_candicates=type_canditates,
                prelude=p,wrapper=w,smt=sm,toString=s,setupCode=sc,
                cod_term_tab=ctt,     abstest_data_tab=tdt,
		dataconvCode=dc,
                test_data_tab=tjt,pre_safe_tac=pst,pre_normalizeTNF_tac=pnt,pre_minimizeTNF_tac=pmt});

fun pre_safe_tac_update (pre_safe_tat)
       (Testenv{depth=dd, breadth=db,
                test_thm_tab=tttab,   bound=b,
                case_breadth=cb,      iterations=i,
                type_range_bound=trb, type_candicates=tc,prelude=p,wrapper=w,smt=sm,toString=s,setupCode=sc,
                cod_term_tab=ctt,     abstest_data_tab=tdt,
		dataconvCode=dc,
                test_data_tab=tjt,pre_safe_tac=pst,pre_normalizeTNF_tac=pnt,pre_minimizeTNF_tac=pmt})
        =
       (Testenv{depth=dd,     breadth=db,
                test_thm_tab=tttab,   bound=b,
                case_breadth=cb,      iterations=i,
                type_range_bound=trb, type_candicates=tc,
                prelude=p,wrapper=w,smt=sm,toString=s,setupCode=sc,
                cod_term_tab=ctt,     abstest_data_tab=tdt,
		dataconvCode=dc,
                test_data_tab=tjt,pre_safe_tac=pre_safe_tat,pre_normalizeTNF_tac=pnt,pre_minimizeTNF_tac=pmt});


fun gen_prelude_update (pre)
       (Testenv{depth=dd, breadth=db,
                test_thm_tab=tttab,   bound=b,
                case_breadth=cb,      iterations=i,
                type_range_bound=trb, type_candicates=tc,prelude=p,wrapper=w,smt=sm,toString=s,setupCode=sc,
                cod_term_tab=ctt,     abstest_data_tab=tdt,
		dataconvCode=dc,
                test_data_tab=tjt,pre_safe_tac=pst,pre_normalizeTNF_tac=pnt,pre_minimizeTNF_tac=pmt})
        =
       (Testenv{depth=dd,     breadth=db,
                test_thm_tab=tttab,   bound=b,
                case_breadth=cb,      iterations=i,
                type_range_bound=trb, type_candicates=tc,prelude=pre,wrapper=w,smt=sm,toString=s,setupCode=sc,
                cod_term_tab=ctt,     abstest_data_tab=tdt,
		dataconvCode=dc,
                test_data_tab=tjt,pre_safe_tac=pst,pre_normalizeTNF_tac=pnt,pre_minimizeTNF_tac=pmt});

fun gen_wrapper_update (wrap)
       (Testenv{depth=dd, breadth=db,
                test_thm_tab=tttab,   bound=b,
                case_breadth=cb,      iterations=i,
                type_range_bound=trb, type_candicates=tc,prelude=p,wrapper=w,
		smt=sm,toString=s,setupCode=sc,
		dataconvCode=dc,
                cod_term_tab=ctt,     abstest_data_tab=tdt,
                test_data_tab=tjt,pre_safe_tac=pst,pre_normalizeTNF_tac=pnt,pre_minimizeTNF_tac=pmt})
        =
       (Testenv{depth=dd,     breadth=db,
                test_thm_tab=tttab,   bound=b,
                case_breadth=cb,      iterations=i,
                type_range_bound=trb, type_candicates=tc,prelude=p,
		wrapper=wrap, smt=sm,toString=s,setupCode=sc,
                cod_term_tab=ctt,     abstest_data_tab=tdt,
		dataconvCode=dc,
                test_data_tab=tjt,pre_safe_tac=pst,pre_normalizeTNF_tac=pnt,pre_minimizeTNF_tac=pmt});

fun SMT_update (sm')
       (Testenv{depth=dd, breadth=db,
                test_thm_tab=tttab,   bound=b,
                case_breadth=cb,      iterations=i,
                type_range_bound=trb, type_candicates=tc,prelude=p,wrapper=w,
		smt=sm,toString=s,setupCode=sc,
		dataconvCode=dc,
                cod_term_tab=ctt,     abstest_data_tab=tdt,
                test_data_tab=tjt,pre_safe_tac=pst,pre_normalizeTNF_tac=pnt,pre_minimizeTNF_tac=pmt})
        =
       (Testenv{depth=dd,     breadth=db,
                test_thm_tab=tttab,   bound=b,
                case_breadth=cb,      iterations=i,
                type_range_bound=trb, type_candicates=tc,prelude=p,
		wrapper=w, smt=sm',toString=s,setupCode=sc,
                cod_term_tab=ctt,     abstest_data_tab=tdt,
		dataconvCode=dc,
                test_data_tab=tjt,pre_safe_tac=pst,pre_normalizeTNF_tac=pnt,pre_minimizeTNF_tac=pmt});


fun gen_toString_update (str)
       (Testenv{depth=dd, breadth=db,
                test_thm_tab=tttab,   bound=b,
                case_breadth=cb,      iterations=i,
                type_range_bound=trb, type_candicates=tc,prelude=p,wrapper=w,smt=sm,
		toString=s,setupCode=sc,
		dataconvCode=dc,
                cod_term_tab=ctt,     abstest_data_tab=tdt,
                test_data_tab=tjt,pre_safe_tac=pst,pre_normalizeTNF_tac=pnt,pre_minimizeTNF_tac=pmt})
        =
       (Testenv{depth=dd,     breadth=db,
                test_thm_tab=tttab,   bound=b,
                case_breadth=cb,      iterations=i,
                type_range_bound=trb, type_candicates=tc,prelude=p,
		wrapper=w, smt=sm,toString=str,setupCode=sc,
                cod_term_tab=ctt,     abstest_data_tab=tdt,
		dataconvCode=dc,
                test_data_tab=tjt,
                pre_safe_tac=pst,pre_normalizeTNF_tac=pnt,pre_minimizeTNF_tac=pmt});


fun setup_code_update (str)
       (Testenv{depth=dd, breadth=db,
                test_thm_tab=tttab,   bound=b,
                case_breadth=cb,      iterations=i,
                type_range_bound=trb, type_candicates=tc,prelude=p,wrapper=w,
		smt=sm,toString=s,setupCode=sc,
		dataconvCode=dc,
                cod_term_tab=ctt,     abstest_data_tab=tdt,
                test_data_tab=tjt,
                pre_safe_tac=pst,pre_normalizeTNF_tac=pnt,pre_minimizeTNF_tac=pmt})
        =
       (Testenv{depth=dd,     breadth=db,
                test_thm_tab=tttab,   bound=b,
                case_breadth=cb,      iterations=i,
                type_range_bound=trb, type_candicates=tc,prelude=p,
		wrapper=w,smt=sm, toString=s,setupCode=str,
                cod_term_tab=ctt,     abstest_data_tab=tdt,
		dataconvCode=dc,
                test_data_tab=tjt,
                pre_safe_tac=pst,pre_normalizeTNF_tac=pnt,pre_minimizeTNF_tac=pmt});

fun dataconv_code_update (str)
       (Testenv{depth=dd, breadth=db,
                test_thm_tab=tttab,   bound=b,
                case_breadth=cb,      iterations=i,
                type_range_bound=trb, type_candicates=tc,prelude=p,wrapper=w,
		smt=sm,toString=s,setupCode=sc,
		dataconvCode=dc,
                cod_term_tab=ctt,     abstest_data_tab=tdt,
                test_data_tab=tjt,pre_safe_tac=pst,pre_normalizeTNF_tac=pnt,pre_minimizeTNF_tac=pmt})
        =
       (Testenv{depth=dd,     breadth=db,
                test_thm_tab=tttab,   bound=b,
                case_breadth=cb,      iterations=i,
                type_range_bound=trb, type_candicates=tc,prelude=p,
		wrapper=w, smt=sm,toString=s,dataconvCode=str,
		setupCode=sc,
                cod_term_tab=ctt,     abstest_data_tab=tdt,
                test_data_tab=tjt,pre_safe_tac=pst,pre_normalizeTNF_tac=pnt,pre_minimizeTNF_tac=pmt});

fun pre_normalizeTNF_tac_update (pre_normalizeTNF_tat)
       (Testenv{depth=dd, breadth=db,
                test_thm_tab=tttab,   bound=b,
                case_breadth=cb,      iterations=i,
                type_range_bound=trb, type_candicates=tc,prelude=p,wrapper=w,smt=sm,toString=s,setupCode=sc,
                cod_term_tab=ctt,     abstest_data_tab=tdt,
		dataconvCode=dc,
                test_data_tab=tjt,pre_safe_tac=pst,pre_normalizeTNF_tac=pnt,pre_minimizeTNF_tac=pmt})
        =
       (Testenv{depth=dd,     breadth=db,
                test_thm_tab=tttab,   bound=b,
                case_breadth=cb,      iterations=i,
                type_range_bound=trb, type_candicates=tc,
                prelude=p,wrapper=w,smt=sm,toString=s,setupCode=sc,
                cod_term_tab=ctt,     abstest_data_tab=tdt,
		dataconvCode=dc,
                test_data_tab=tjt,pre_safe_tac=pst,pre_normalizeTNF_tac=pre_normalizeTNF_tat,
                pre_minimizeTNF_tac=pmt});


fun pre_minimizeTNF_tac_update (pre_minimizeTNF_tat)
       (Testenv{depth=dd, breadth=db,
                test_thm_tab=tttab,   bound=b,
                case_breadth=cb,      iterations=i,
                type_range_bound=trb, type_candicates=tc,prelude=p,wrapper=w,smt=sm,toString=s,setupCode=sc,
                cod_term_tab=ctt,     abstest_data_tab=tdt,
		dataconvCode=dc,
                test_data_tab=tjt,pre_safe_tac=pst,pre_normalizeTNF_tac=pnt,
                pre_minimizeTNF_tac=pmt})
        =
       (Testenv{depth=dd,     breadth=db,
                test_thm_tab=tttab,   bound=b,
                case_breadth=cb,      iterations=i,
                type_range_bound=trb, type_candicates=tc,
                prelude=p,wrapper=w,smt=sm,toString=s,setupCode=sc,
                cod_term_tab=ctt,     abstest_data_tab=tdt,
		dataconvCode=dc,
                test_data_tab=tjt,pre_safe_tac=pst,pre_normalizeTNF_tac=pnt,
                pre_minimizeTNF_tac=pre_minimizeTNF_tat});




fun get_depth (Testenv{depth=dd, ...}) = dd;

fun get_breadth (Testenv{breadth=db, ...}) = db;

fun get_gen_prelude (Testenv{prelude=p, ...}) = p;
fun get_gen_wrapper (Testenv{wrapper=w, ...}) = w;
fun get_SMT (Testenv{smt=s, ...}) = s;
fun get_gen_toString (Testenv{toString=w, ...}) = w;
fun get_setup_code (Testenv{setupCode=w, ...}) = w;
fun get_dataconv_code (Testenv{dataconvCode=w, ...}) = w;
fun get_pre_safe_tac (Testenv{pre_safe_tac=pst, ...}) = pst;
fun get_pre_normalizeTNF_tac (Testenv{pre_normalizeTNF_tac=pnt, ...}) = pnt;
fun get_pre_minimizeTNF_tac (Testenv{pre_minimizeTNF_tac=pmt, ...}) = pmt;

fun export_test_data thy filename dataname te = let
    val thms        = PureThy.get_thms thy (dataname^".test_data")
    val margin = ref 78;
    (*    val thms strings = Pretty.string_of (Display.pretty_thms thms) *)
    val today = (Date.toString(Date.fromTimeUniv (Time.now())))^" (UTC)";
    val thyname = Context.theory_name thy
    fun generate (thmlist:(thm list)) = let
	val num = Int.toString(length thms) 
	val thmsstrings = String.concat (map (fn t => "<testcase>\n"
						    ^((Display.string_of_cterm o cprop_of) t)^"\n"
						    ^"</testcase>\n"
						    ) thms)
    in
	"<!---------------------------------------------------------------------\n"
	^" --       2008 Test-Data ("^thyname^"."^dataname^", "^num^" test cases)\n"
	^" --           generated by HOL-TestGen "^testgen_version^"\n"
	^" --           generated on "^today^"\n"
	^" --------------------------------------------------------------------->\n\n"
	^thmsstrings
    end
					
    val test_data_str  = setmp print_mode [] (Pretty.setmp_margin (!margin) (generate)) thms
    val _  = File.write (Path.explode filename) test_data_str;	

in
    thy
end
    
end (*struct *);
