(*****************************************************************************
 * HOL-TestGen --- theorem-prover based test case generation
 *                 http://www.brucker.ch/projects/hol-testgen/
 *                                                                            
 * isar_setup.sml --- Isar setup for HOL-TestGen
 * This file is part of HOL-TestGen.
 *
 * Copyright (c) 2005-2007 ETH Zurich, Switzerland
 *               2008 Achim D. Brucker, Germany
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *
 *     * Redistributions in binary form must reproduce the above
 *       copyright notice, this list of conditions and the following
 *       disclaimer in the documentation and/or other materials provided
 *       with the distribution.
 *
 *     * Neither the name of the copyright holders nor the names of its
 *       contributors may be used to endorse or promote products derived
 *       from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ******************************************************************************)
(* $Id: isar_setup.ML 8855 2010-03-24 13:04:06Z krieger $ *)

(* at the moment, calling store_test_thm without the optional argument  works only 
   in an ProofGeneral session because otherweise the call of Toplevel.pretty_state 
   results in an empty list \<dots> 
*)
structure P = OuterParse and K = OuterKeyword;

			     

val auto_quickcheck_save = ref(true)
(* This mechanism is slightly buggy since it assumes that any test_spec
   is closed by a store_test_thm command (as there are in standard situations).
   However, there is the case of "oops" etc. too... *)

fun store_test_thm name = Toplevel.end_proof (fn _ => fn state =>
  let
      val _ = (auto_quickcheck:=(!auto_quickcheck_save));
    val _ = if !TestGen.profiling_on then 
		(Clocks.start_clock "unnamed_test_thm"; 
		 Clocks.rename_clock "unnamed_test_thm" name; 
		 Clocks.stop_clock name) 
	    else 
		()

    val _ = Proof.assert_backward state;
    val (goal_ctxt, (_, goal)) = Proof.get_goal state
    fun up_thy thy = 
	let 	  
	  fun add_to_dm name thm thy = 
	      let
		val ctx = Context.Theory thy 
		val dm  = TestGen_DataManagement.get(ctx);
		val dm  = TestEnv.add_test_case (dm, (name,thm))
		val ctx = TestGen_DataManagement.put dm ctx
                val t = LogThy.get_tc_delta ()
		val num_of_tc = List.length 
				(List.filter (fn tc => not (TestGen.is_thyp tc)) (prems_of thm))
		val _ = print (concat ["Test theorem (gen_test_cases) '",name,"': ", 
				       Int.toString num_of_tc, 
				       " test cases in ",Time.toString t, " seconds"])

		val _ = LogThy.append (concat [Context.theory_name thy, ", "
				    ,name, ", " 
				    ,"test case, " 
				    ,Int.toString num_of_tc,", " 
				    , Time.toString t,"\n"])
              in
		Context.theory_of ctx
	      end
	in
	  thy |> Sign.add_path (space_implode "_" [name])
	      |> (PureThy.add_thms [((Binding.name("test_thm"), goal), [])])
	      |> snd
	      |> Sign.parent_path
	      |> add_to_dm (space_implode "_" [name]) goal
	      |> Clocks.flush_clocks
	end

  in goal_ctxt |> LocalTheory.theory up_thy  end);

val _ =
  OuterSyntax.command "store_test_thm" "store test state (theorem)"
    (OuterKeyword.tag_proof OuterKeyword.qed)        
    (OuterParse.name >> store_test_thm);   




    
(**********************)
fun gen_test_dataT name thyctxt = 
    let 
                val thy = thyctxt
	fun gen_test_data name   = 
	    let
		val dm = TestGen_DataManagement.get(Context.Theory thy);
		val _  = LogThy.start_td_timer ()
		val _ = if !TestGen.profiling_on then 
			    (Clocks.init_clocks thy;
			     Clocks.start_clock name;
			     Clocks.start_clock "gen_test_data")
			else 
			    ()
		
		val dm = TestGen.gen_test_data thy  (claset_of(thy),simpset_of(thy)) 
					       name dm; 
                val t = LogThy.get_td_delta ()
		val thy = if !TestGen.profiling_on then 
			      (Clocks.stop_clock "gen_test_data";
			       Clocks.stop_clock name;
			       Clocks.flush_clocks thy)
			  else 
			      thy

		val thy =  Context.theory_of 
			       (TestGen_DataManagement.put dm (Context.Theory thy))
		val data = TestGen.get_test_data (dm,name)
		val hyps = TestGen.get_test_hyps (dm,name); 
		val thy  = Sign.add_path (space_implode "_" [name]) thy;
                val thy = snd(PureThy.add_thmss [((Binding.name("test_hyps"),hyps),[])] (thy)); 
		val thy = snd(PureThy.add_thmss [((Binding.name("test_data"),data),[])] (thy));
		val _ = print (concat ["Test theorem (gen_test_data) '",name,"': ", 
				       Int.toString (List.length data), 
				       " test cases in ",Time.toString t, " seconds"])

		val _ = LogThy.append (concat [Context.theory_name thy, ", "
				    ,name, ", " 
				    ,"test data, "
				    ,Int.toString (List.length data),", " 
				    ,Time.toString t,"\n"])

                val thy = Sign.parent_path thy;
	    in 
		thy
	    end
        val thy = gen_test_data name 
    in 
        thy
    end 
    
val gen_test_dataP =
    OuterSyntax.command "gen_test_data" "generate test data" 
			OuterKeyword.thy_script
			((P.name)  >> (Toplevel.theory o gen_test_dataT));
    
(**********************)

fun gen_testscriptT (((filename,name),prog),smlprogname) thy  = 
	let 
	  val te = TestGen_DataManagement.get(Context.Theory thy);
	  val thy = TestGen.gen_testscript thy prog smlprogname filename name te;		
	in 
		thy 
	end

	
val gen_test_dataP =
    OuterSyntax.command "gen_test_script" "generate test script" 
			OuterKeyword.thy_script
			((P.name  -- P.name  -- P.name -- Scan.optional P.name "")
				>> (fn (((filename,name),prog),smlprogname) =>
					(Toplevel.theory (fn thy => gen_testscriptT (((filename,name),prog),smlprogname) thy ))));
(**********************)

(**********************)

fun export_test_dataT (filename,name) thy  = 
	let 
	  val te = TestGen_DataManagement.get(Context.Theory thy);
	  val thy = TestEnv.export_test_data thy filename name te;		
	in 
		thy 
	end

	
val export_test_dataP =
    OuterSyntax.command "export_test_data" "export test data to an external file" 
			OuterKeyword.thy_script
			((P.name  -- P.name)
				>> (fn (filename,name) =>
					(Toplevel.theory (fn thy => export_test_dataT (filename,name) thy ))));
(**********************)
    

val locale_target = (P.$$$ "(" -- P.$$$ "in") |-- P.!!! (P.xname --| P.$$$ ")");
val opt_locale_target = Scan.option locale_target;



fun gen_theorem kind =
  OuterSyntax.command kind ("state " ^ kind) K.thy_goal
    (P.opt_target -- Scan.optional (SpecParse.opt_thm_name ":" --|
      Scan.ahead (SpecParse.locale_keyword || SpecParse.statement_keyword)) (Binding.name(""), []) --
      SpecParse.general_statement >> (fn ((loc, a), (elems, concl)) =>
      (let val _ = (auto_quickcheck_save := (!auto_quickcheck));
           val _ = (auto_quickcheck:=false)

	   fun theorem_cmd b ctxt = let
	       val _ = if !TestGen.profiling_on then Clocks.init_clocks (ProofContext.theory_of ctxt) else ()
	   in
	       (Specification.theorem_cmd kind NONE (K I) a elems concl) b ctxt
	   end

       in  Toplevel.print o Toplevel.local_theory_to_proof' loc theorem_cmd

       end)));

val _ = gen_theorem "test_spec"




fun update_typlist sl (sg) (x:TestEnv.testenv) = 
let
    val typlist =  map (fn ts => (Syntax.read_typ (ProofContext.init sg) ts)) sl;
in 
    TestEnv.type_candiates_update typlist x
end

fun update_gen_prelude value (x:TestEnv.testenv) = 
let
    val prelude = if (value = "true") then true else false
in 
    TestEnv.gen_prelude_update prelude x
end

fun update_gen_wrapper value (x:TestEnv.testenv) = 
let
    val wrapper = if (value = "true") then true else false
in 
    TestEnv.gen_wrapper_update wrapper x
end

fun update_SMT value (x:TestEnv.testenv) = 
let
    val smt = if (value = "true") then true else false
in 
    TestEnv.SMT_update smt x
end

fun map_testgen_params (f:(TestEnv.testenv -> TestEnv.testenv)) thy =
  let 
      val dm = TestGen_DataManagement.get(Context.Theory thy);
      val dm = f dm;   
  in 
      Context.theory_of (TestGen_DataManagement.put dm (Context.Theory thy))
  end

fun app [] x = x
  | app (f :: fs) x = app fs (f x);


val testgen_params =
  [("depth", P.nat >> (K o TestEnv.depth_update)),
   ("breadth", P.nat >> (K o TestEnv.breadth_update)),
   ("bound", P.nat >> (K o TestEnv.bound_update)),
   ("case_breadth", P.nat >> (K o TestEnv.case_breadth_update)),
   ("iterations", P.nat >> (K o TestEnv.iterations_update)),
   ("gen_prelude", P.name >> (K o update_gen_prelude)),
   ("gen_wrapper", P.name >> (K o update_gen_wrapper)),
   ("SMT", P.name >> (K o update_SMT)),
   ("toString", P.name >> (K o TestEnv.gen_toString_update)),
   ("setup_code", P.name >> (K o TestEnv.setup_code_update)),
   ("dataconv_code", P.name >> (K o TestEnv.dataconv_code_update)),
   ("type_range_bound", P.nat >> (K o TestEnv.type_range_bound_update)), 
   ("type_candidates",(P.$$$ "[" |-- P.list1 P.typ --| P.$$$ "]") >> update_typlist)
 
];



fun assoc(S,x) = case Library.find_first(fn(y,_)=> x=y)S of
                   SOME(x,y) => SOME(y)
		  |NONE      => NONE;

fun if_none NONE y = y
  | if_none (SOME x) _ = x;


val parse_testgen_params = P.short_ident :-- (fn s =>
  P.$$$ "=" |-- if_none (assoc (testgen_params, s)) Scan.fail) >> snd;

val testgen_paramsP =
  OuterSyntax.command "testgen_params" "set parameters for TestGen" K.thy_decl
    (P.$$$ "[" |-- P.list1 parse_testgen_params --| P.$$$ "]" >> 
     (fn fs => Toplevel.theory (fn thy =>
         map_testgen_params (app (map (fn f => f (thy )) fs)) thy)));



(**********************)
fun testgen_add name (gctx, thm) = 
    let
	val dm   = TestGen_DataManagement.get(gctx); 
	val dm   = TestEnv.add_test_data (dm, (name,thm));  
	val gctx = TestGen_DataManagement.put dm (gctx); 
    in 
	(gctx, thm)
    end

(**********************)

fun test_name x = Attrib.syntax (Scan.lift Args.name >> testgen_add) x;


val TestingIsar_setup = Attrib.add_attributes [("test",
			(test_name (*, 
			 K Attrib.undef_local_attribute *)),
			"declare theorems for test case generation")];


fun profiling_on_command thy = (TestGen.profiling_on := true; thy)

val _ = OuterSyntax.command "profiling_on" "turns on profiling of the test procedure" 
			    OuterKeyword.thy_decl (Scan.succeed (Toplevel.theory profiling_on_command));

fun profiling_off_command thy = (TestGen.profiling_on := false; thy)

val _ = OuterSyntax.command "profiling_off" "turns off profiling of the test procedure" 
			    OuterKeyword.thy_decl (Scan.succeed (Toplevel.theory profiling_off_command));